@@ -23,6 +23,7 @@ import random
 from itertools import combinations
 import matplotlib.pyplot as plt
 import json
+import ipywidgets as widgets
 
 
 class Network(object):
@@ -39,8 +40,6 @@ class Network(object):
   '''
 
   def __init__(self, widget=None):
-
-    # print('INITIALIZING')
     initialize_net.main(self, widget)
 
   def reset(self):
@@ -91,7 +90,8 @@ class Network(object):
   def cluster(self, dist_type='cosine', run_clustering=True,
                  dendro=True, views=[],
                  linkage_type='average', sim_mat=False, filter_sim=0.0,
-                 calc_cat_pval=False, run_enrichr=None, enrichrgram=None):
+                 calc_cat_pval=False, run_enrichr=None, enrichrgram=None,
+                 clust_library='scipy', min_samples=1, min_cluster_size=2):
     '''
     The main function performs hierarchical clustering, optionally generates
     filtered views (e.g. row-filtered views), and generates the :
@@ -110,7 +110,10 @@ class Network(object):
                                     filter_sim=filter_sim,
                                     calc_cat_pval=calc_cat_pval,
                                     run_enrichr=run_enrichr,
-                                    enrichrgram=enrichrgram)
+                                    enrichrgram=enrichrgram,
+                                    clust_library=clust_library,
+                                    min_samples=min_samples,
+                                    min_cluster_size=min_cluster_size)
 
   def swap_nan_for_zero(self):
     '''
@@ -118,6 +121,65 @@ class Network(object):
     '''
     self.dat['mat'][np.isnan(self.dat['mat'])] = 0
 
+  # # alternative approach
+  # ########################
+  # def load_meta(self, col=None, row=None,
+  #   col_cats=None, row_cats=None,
+  #   is_downsampled=False,
+  #   ds_row=None, ds_col=None):
+
+  #   if is_downsampled:
+  #     if meta_ds_col is not None:
+  #       self.meta_ds_col = meta_ds_col
+  #     if meta_ds_row is not None:
+  #       self.meta_ds_row = meta_ds_row
+
+  #   # define downsampled status
+  #   self.is_downsampled = is_downsampled
+  #   # print('load_df: is_downsampled', is_downsampled)
+
+  #   if hasattr(self, 'meta_col') == False and hasattr(self, 'meta_row') == False:
+  #     self.meta_cat = False
+
+  #   # load metadata
+  #   if isinstance(col, pd.DataFrame):
+  #     self.meta_col = col
+
+  #     if col_cats is None:
+  #       self.col_cats = col.columns.tolist()
+  #     else:
+  #       self.col_cats = col_cats
+
+  #     self.meta_cat = True
+
+  #   if isinstance(row, pd.DataFrame):
+  #     self.meta_row = row
+
+  #     if row_cats is None:
+  #       self.row_cats = row.columns.tolist()
+  #     else:
+  #       self.row_cats = row_cats
+
+  #     self.meta_cat = True
+
+  # def load_df(self, df_ini, col_cats=None, row_cats=None):
+  #   '''
+  #   Load Pandas DataFrame and assign categories
+  #   '''
+  #   self.reset()
+
+  #   # load dataframe
+  #   df = deepcopy(df_ini)
+
+  #   # load specify categories for visualization
+  #   if col_cats is not None:
+  #     self.col_cats = col_cats
+
+  #   if row_cats is not None:
+  #     self.row_cats = row_cats
+
+  #   data_formats.df_to_dat(self, df, define_cat_colors=True)
+
   def load_df(self, df_ini, meta_col=None, meta_row=None, col_cats=None,
               row_cats=None, is_downsampled=False, meta_ds_row=None,
               meta_ds_col=None):
@@ -129,8 +191,6 @@ class Network(object):
     # load dataframe
     df = deepcopy(df_ini)
 
-    if hasattr(self, 'meta_col') == False and hasattr(self, 'meta_row') == False:
-      self.meta_cat = False
 
     if is_downsampled:
       if meta_ds_col is not None:
@@ -142,6 +202,9 @@ class Network(object):
     self.is_downsampled = is_downsampled
     # print('load_df: is_downsampled', is_downsampled)
 
+    if hasattr(self, 'meta_col') == False and hasattr(self, 'meta_row') == False:
+      self.meta_cat = False
+
     # load metadata
     if isinstance(meta_col, pd.DataFrame):
       self.meta_col = meta_col
@@ -190,6 +253,13 @@ class Network(object):
     self.viz['matrix_colors']['pos'] = pos
     self.viz['matrix_colors']['neg'] = neg
 
+  def set_global_cat_colors(self, df_meta):
+
+    for inst_name in df_meta.index.tolist():
+      inst_color = df_meta.loc[inst_name, 'color']
+
+      self.viz['global_cat_colors'][inst_name] = inst_color
+
   def set_cat_color(self, axis, cat_index, cat_name, inst_color):
 
     if axis == 0:
@@ -227,7 +297,8 @@ class Network(object):
 
     return export_data.export_net_json(self, which_viz, 'no-indent')
 
-  def widget(self, which_viz='viz'):
+  def widget(self, which_viz='viz', link_net=None, clust_library='scipy',
+    min_samples=1, min_cluster_size=2):
     '''
     Generate a widget visualization using the widget. The export_viz_to_widget
     method passes the visualization JSON to the instantiated widget, which is
@@ -235,7 +306,8 @@ class Network(object):
     '''
     # run clustering if necessary
     if len(self.viz['row_nodes']) == 0:
-      self.cluster()
+      self.cluster(clust_library=clust_library, min_samples=min_samples,
+                   min_cluster_size=min_cluster_size)
 
       # add manual_category to viz json
       if 'manual_category' in self.dat:
@@ -247,6 +319,29 @@ class Network(object):
 
     self.widget_instance = self.widget_class(network = self.export_viz_to_widget(which_viz))
 
+    # initialize manual category
+    if 'manual_category' in self.dat:
+      manual_cat = {}
+      axis = 'col'
+      manual_cat[axis] = {}
+      manual_cat[axis]['col_cat_colors'] = self.viz['cat_colors'][axis]['cat-0']
+
+      man_cat_name = self.dat['manual_category'][axis]
+      if axis == 'col':
+        manual_cat[axis][man_cat_name] = self.meta_col[man_cat_name].to_dict()
+
+      self.widget_instance.manual_cat = json.dumps(manual_cat)
+
+      self.widget_instance.observe(self.get_manual_category, names='manual_cat')
+
+    # add link
+    if link_net is not None:
+      inst_link = widgets.link(
+                                (self.widget_instance, 'manual_cat'),
+                                (link_net.widget_instance, 'manual_cat')
+                               )
+      self.widget_instance.link = inst_link
+
     return self.widget_instance
 
 
@@ -373,11 +468,11 @@ class Network(object):
     df = df.clip(lower=lower, upper=upper)
     self.load_df(df)
 
-  def normalize(self, df=None, norm_type='zscore', axis='row'):
+  def normalize(self, df=None, norm_type='zscore', axis='row', z_clip=None):
     '''
     Normalize the matrix rows or columns using Z-score (zscore) or Quantile Normalization (qn). Users can optionally pass in a DataFrame to be normalized (and this will be incorporated into the Network object).
     '''
-    normalize_fun.run_norm(self, df, norm_type, axis)
+    normalize_fun.run_norm(self, df, norm_type, axis, z_clip)
 
   def downsample(self, df=None, ds_type='kmeans', axis='row', num_samples=100,
                  random_state=1000, ds_name='Downsample',
@@ -707,6 +802,9 @@ class Network(object):
 
       ''' Generate signatures for column categories '''
 
+      # change this to use category name and set caetgory level to 1 always
+      ###################################
+
       df_t = df_ini.transpose()
 
       # remove columns with constant values
@@ -1173,7 +1271,8 @@ class Network(object):
         # update manual category
         self.meta_col.loc[found_labels, man_cat_title] = inst_man_cat
 
-  def get_manual_category(self):
+  def get_manual_category(self, tmp):
+
 
     for axis in ['col']:
 
@@ -1183,6 +1282,7 @@ class Network(object):
         export_dict = {}
 
         inst_nodes = self.dat['nodes'][axis]
+        # print('get get_manual_category', len(inst_nodes))
 
         cat_title = self.dat['manual_category'][axis]
 
@@ -1190,9 +1290,7 @@ class Network(object):
         try:
         ###########################################################
 
-
-          export_dict[cat_title] = pd.Series(json.loads(self.widget_instance.custom_cat)[axis][cat_title])
-
+          export_dict[cat_title] = pd.Series(json.loads(self.widget_instance.manual_cat)[axis][cat_title])
 
           if hasattr(self, 'meta_cat') == True:
 
@@ -1205,32 +1303,48 @@ class Network(object):
 
             elif axis == 'col':
               if self.is_downsampled == False:
-                self.meta_col.loc[inst_nodes, cat_title] = export_dict[cat_title]
+                # print('> > > > > > > > > > > ')
+                # print(export_dict[cat_title])
+
+                # print('.. .. .. .. .. .. .. .. .. .. .. .. ')
+                # print(self.meta_col)
+
+                # # for some reason this is breaking when trying to sync metadata
+                # # switching to slower row based syncing
+                # self.meta_col.loc[inst_nodes, cat_title] = export_dict[cat_title]
+
+                # manually looping through rows in metadata works
+                for inst_row in export_dict[cat_title].index.tolist():
+                  self.meta_col.loc[inst_row, cat_title] = export_dict[cat_title][inst_row]
+
+                # print('>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>')
+                # print(self.meta_col)
+
               else:
                 self.meta_ds_col.loc[inst_nodes, cat_title] = export_dict[cat_title]
                 self.ds_to_original_meta(axis)
 
 
         except:
-          pass
-          # print('unable to load', axis ,' category, please check title')
+          print('unable to load', axis ,' category, please check title')
 
         # Category Colors
         #######################
-        ini_new_colors = json.loads(self.widget_instance.custom_cat)[axis + '_cat_colors']
-        # drop title from category colors
-        export_dict['cat_colors'] = {}
-        for inst_cat in ini_new_colors:
-          if (': ' in inst_cat):
-            export_dict['cat_colors'][inst_cat.split(': ')[1]] = ini_new_colors[inst_cat]
-          else:
-            export_dict['cat_colors'][inst_cat] = ini_new_colors[inst_cat]
+        export_dict['cat_colors'] = json.loads(self.widget_instance.manual_cat)['global_cat_colors']
 
-        if hasattr(self, 'meta_cat') == False:
-          return export_dict
+        # # drop title from category colors
+        # export_dict['cat_colors'] = {}
+        # for inst_cat in ini_new_colors:
+        #   if (': ' in inst_cat):
+        #     export_dict['cat_colors'][inst_cat.split(': ')[1]] = ini_new_colors[inst_cat]
+        #   else:
+        #     export_dict['cat_colors'][inst_cat] = ini_new_colors[inst_cat]
+
+        # if hasattr(self, 'meta_cat') == False:
+        #   return export_dict
 
       except:
-          # print('please set custom category')
+          # print('failed to parse manual_category')
           pass
 
 
@@ -1261,7 +1375,7 @@ class Network(object):
         clean_cols.append(tuple(inst_clean))
 
     df_ini = pd.DataFrame(data=clean_cols).set_index(0)
-    mat = df_ini.get_values()
+    mat = df_ini.values
     rows = df_ini.index.tolist()
 
     df_meta = pd.DataFrame(data=mat, index=rows, columns=cat_titles)
