@@ -8,8 +8,6 @@ def export_net_json(net, net_type, indent='no-indent'):
 
     if type(exp_dict['mat']) is not list:
       exp_dict['mat'] = exp_dict['mat'].tolist()
-      if 'mat_orig' in exp_dict:
-        exp_dict['mat_orig'] = exp_dict['mat_orig'].tolist()
 
   elif net_type == 'viz':
     exp_dict = net.viz
@@ -39,7 +37,7 @@ def write_matrix_to_tsv(net, filename=None, df=None):
   if df is None:
     df = net.dat_to_df()
 
-  return df['mat'].to_csv(filename, sep='\t')
+  return df.to_csv(filename, sep='\t')
 
 def write_json_to_file(net, net_type, filename, indent='no-indent'):
 
