@@ -13,7 +13,7 @@ from traitlets import Unicode
 from ._frontend import module_name, module_version
 
 
-class ExampleWidget(DOMWidget):
+class CGM2(DOMWidget):
     """TODO: Add docstring here
     """
     _model_name = Unicode('ExampleModel').tag(sync=True)
@@ -24,10 +24,10 @@ class ExampleWidget(DOMWidget):
     _view_module_version = Unicode(module_version).tag(sync=True)
 
     value = Unicode('Hello World!').tag(sync=True)
-    custom_cat = Unicode('{"col":"", "row":""}').tag(sync=True)
+    manual_cat = Unicode('{"col":"", "row":""}').tag(sync=True)
 
     mat_string = Unicode('ini_mat_string').tag(sync=True)
 
-    print('>> clustergrammer2 backend version 0.13.2')
+    print('>> clustergrammer2 backend version 0.15.1')
 
     network = Unicode('').tag(sync=True)
\ No newline at end of file
