@@ -14,7 +14,7 @@ import cgm_fun from 'clustergrammer-gl';
 import * as d3 from 'd3';
 
 console.log('*********************************************')
-console.log('** clustergrammer2 frontend version 0.13.2 **')
+console.log('** clustergrammer2 frontend version 0.15.1 **')
 console.log('*********************************************')
 
 export
@@ -28,7 +28,7 @@ class ExampleModel extends DOMWidgetModel {
       _view_module: ExampleModel.view_module,
       _view_module_version: ExampleModel.view_module_version,
       value : 'javascript set value!!!',
-      custom_cat: '{"col":"JS", "row":"JS"}',
+      manual_cat: '{"col":"JS", "row":"JS"}',
       network: ''
     };
   }
@@ -120,8 +120,6 @@ var my_widget_callback = function(external_model){
       // look up indexes in original dataframe
       for (let inst_name of selected_clust_names) {
 
-        // console.log(inst_name)
-
         // get index of label
         tmp_index = all_labels.indexOf(inst_name)
 
@@ -141,98 +139,37 @@ var my_widget_callback = function(external_model){
       external_model.touch();
 
   }
-  // else {
-
-  //   console.log('OTHER')
-
-  //   inst_value = params.tooltip.tooltip_type; // 'other'
-
-  //   // update other
-  //   ////////////////////////
-  //   // external_model.model.set('value', String(null));
-  //   external_model.model.set('value', inst_value);
-  //   external_model.touch();
-
-  // }
-
-  // let ini_manual_cat_dict = {}
-  // if (params.cat_data.manual_cat_dict === '{"col":"", "row":""}'){
-  //   ini_manual_cat_dict = true
-  // } else {
-  //   ini_manual_cat_dict = false
-  // }
-
 
   if (params.int.manual_update_cats){
 
     let export_dict = {}
-    // let inst_dict = {}
 
       // transfer manual categories
     if ('col' in params.cat_data.manual_cat_dict){
 
-      // params.network.col_nodes.forEach((d) => {
-      //   let inst_name
-      //   if (d.name.includes(': ')){
-      //     inst_name = d.name.split(': ')[1]
-      //   } else {
-      //     inst_name = d.name
-      //   }
-
-      //   inst_dict[inst_name] = d['cat-0']
-      // })
-      // export_dict['col'] = inst_dict
-
       export_dict['col'] = params.cat_data.manual_cat_dict.col
-      export_dict['col_cat_colors'] = params.cat_colors.col['cat-0']
+      // export_dict['col_cat_colors'] = params.cat_colors.col['cat-0']
     }
 
     if ('row' in params.cat_data.manual_cat_dict){
 
-      // params.network.row_nodes.forEach((d) => {
-      //   let inst_name
-      //   if (d.name.includes(': ')){
-      //     inst_name = d.name.split(': ')[1]
-      //   } else {
-      //     inst_name = d.name
-      //   }
-
-      //   inst_dict[inst_name] = d['cat-0']
-      // })
-      // export_dict['row'] = inst_dict
-
       export_dict['row'] = params.cat_data.manual_cat_dict.row
-      export_dict['row_cat_colors'] = params.cat_colors.row['cat-0']
+      // export_dict['row_cat_colors'] = params.cat_colors.row['cat-0']
     }
 
-    // console.log(export_dict)
+    export_dict['global_cat_colors'] = params.viz.global_cat_colors
 
-    // Object.keys(cgm.params.cat_colors.col['cat-0'])
-
-    // let axes = ['row', 'col']
-    // let cat_colors = {}
-    // let ini_cat_colors = {}
-    // let ini_cats
-
-
-    // if (axis in params.cat_colors){
-    //   ini_cat_colors[axis] = params.cat_colors[axis]['cat-0']
-    // }
-
-    // ini_cats = Object.keys(ini_cat_colors)
-
-    // ini_cats.forEach(x => {
-    //   cat_colors[x.split(': ')[1]] = ini_cat_colors[x]
-    // })
-
-    // // cleaned
-    // console.log('cleaned cat colors')
-    // console.log(cat_colors)
-
-    console.log('>>>>>>>>>>>>>>> manual_update_cats!!!!!!')
     let json_string = JSON.stringify(export_dict)
-    external_model.model.set('custom_cat', json_string);
+    external_model.model.set('manual_cat', json_string);
     external_model.touch();
+
+    // notify manual_cat_update that update is not necessary
+    params.self_update = true
+
+    console.log('>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>')
+    console.log('setting self_update to true')
+    console.log('>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>')
+
   }
 }
 
@@ -261,25 +198,13 @@ class ExampleView extends DOMWidgetView {
 
     this.model.on('change:value', this.value_changed, this);
 
+    this.model.on('change:manual_cat', this.manual_cat_update, this);
+
   }
 
   value_changed() {
 
-    // this can be used to update cgm on a value change to the widget_model
-    // e.g. reorder cgm based on value change to widget traitlet
-
-    // console.log('widget_model.value_changed')
-    // console.log('--------------------------')
-    // console.log(this)
-
-    // console.log('value', this.model.get('value'));
-
-    // // Actually delete the widget and replace with text (not deleting widget state)
-    // this.el.textContent = this.model.get('value');
-
-    // // see https://stackoverflow.com/questions/55834241/ts2339-property-name-does-not-exist-on-type-object
-    // console.log(this['cgm'])
-    // this['cgm'].destroy_viz()
+    console.log('value changed!!!!!!!!!!!!');
 
     if (this.model.get('value') === 'destroy-viz'){
       console.log('destroy the viz')
@@ -288,6 +213,74 @@ class ExampleView extends DOMWidgetView {
 
   }
 
+  manual_cat_update() {
+
+      console.log('***********************************************');
+      console.log('* manual update to cat from backend')
+      console.log('***********************************************');
+
+      let dict_manual_cat = JSON.parse(this.model.get('manual_cat'))
+
+      // console.log(dict_manual_cat)
+      // console.log('getting col from manual_cat')
+      // console.log(dict_manual_cat['col'])
+
+      let manual_category_name = this['cgm']['params']['network']['manual_category']['col']
+      let update_cat_dict = dict_manual_cat['col'][manual_category_name]
+
+      // we don't want to have undefined categories
+      // console.log('for each coming up!')
+      let new_cat_dict = {}
+      this['cgm']['params']['network']['col_nodes']
+          .forEach(x => {
+
+            // assuming no title in name
+            let inst_name = x['name']
+            let orig_cat = x['cat-0'].split(': ')[1]
+
+            // console.log(inst_name, orig_cat)
+
+            if (inst_name in update_cat_dict){
+              new_cat_dict[inst_name] = update_cat_dict[inst_name]
+            } else {
+              new_cat_dict[inst_name] = orig_cat
+            }
+          });
+
+      // console.log('**********************************************')
+      // console.log('**********************************************')
+      // console.log('**********************************************')
+      // console.log('**********************************************')
+      // console.log(new_cat_dict)
+
+      this['cgm']['update_all_cats'](this['cgm'], 'col', manual_category_name, new_cat_dict)
+
+
+    // console.log(this['cgm']);
+
+    // let axis = 'col';
+
+    // let cat_title = this['cgm'].params.network.manual_category.col;
+
+    // this['cgm'].params.network[axis + '_nodes']
+    //     .map(x => {
+
+    //        let new_cat = this['cgm'].params.cat_data.manual_cat_dict[axis];
+
+    //        let full_cat = cat_title + ': ' + new_cat;
+
+    //        let inst_name = x.name;
+
+    //        if ( inst_name.includes(': ') ){
+    //          inst_name = inst_name.split(': ')[1]
+    //        }
+
+    //        x['cat-0'] = full_cat
+
+    //      })
+
+  }
+
 }
 
 
